import { u as useRouter, c as createHref, n as navigate, t as toLocation, a as commit, r as resolve } from './server-1b90876f.mjs';
export { j as HashRouter, H as HistoryRouter, M as MemoryRouter, R as Router, V as View, x as back, y as cancel, q as commitReplace, k as create, i as createRouter, g as defaultResolveView, w as forward, m as getCurrentView, l as getHistoryState, v as go, z as initHistoryStack, A as listen, o as match, s as refresh, h as resolveClientView, p as resolveTo, d as useData, e as useLoading, f as useMatched, b as useView } from './server-1b90876f.mjs';
import { useRef, useState, useEffect, useMemo, createContext, useContext } from 'react';
import { jsx } from 'react/jsx-runtime';
import 'history';
import 'path-to-regexp';

function Link(_ref) {
  let {
    to,
    ...rest
  } = _ref;
  const router = useRouter();
  const lock = useRef(false);
  function handleClick(e) {
    e.preventDefault();
    if (lock.current)
      return;
    lock.current = true;
    navigate(router, to).finally(() => lock.current = false);
  }
  return (
    // eslint-disable-next-line jsx-a11y/anchor-has-content
    /* @__PURE__ */ jsx("a", {
      ...rest,
      href: createHref(router, to),
      onClick: handleClick
    })
  );
}

const Context = /* @__PURE__ */ createContext({
  loading: false
});
function usePrefetch() {
  return useContext(Context);
}
function PrefetchLink(_ref) {
  let {
    to,
    children,
    ...rest
  } = _ref;
  const router = useRouter();
  const viewPromiseRef = useRef();
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState();
  const [view, setView] = useState();
  const location = toLocation(router, to);
  function prefetchIt() {
    setLoading(true);
    viewPromiseRef.current = resolve(router, location).then((v) => {
      setView(v);
      return v;
    }).catch((e) => {
      setError(e);
      throw e;
    }).finally(() => setLoading(false));
  }
  function handlePrefetch() {
    if (viewPromiseRef.current)
      return;
    prefetchIt();
  }
  function handleClick(e) {
    e.preventDefault();
    if (!viewPromiseRef.current) {
      prefetchIt();
    }
    commit(router, viewPromiseRef.current, location).finally(() => viewPromiseRef.current = void 0);
  }
  useEffect(() => {
    viewPromiseRef.current = void 0;
  }, [to, router]);
  const linkContext = useMemo(() => ({
    loading,
    error,
    view
  }), [loading, error, view]);
  return /* @__PURE__ */ jsx(Context.Provider, {
    value: linkContext,
    children: /* @__PURE__ */ jsx("a", {
      ...rest,
      href: createHref(router, to),
      onMouseEnter: handlePrefetch,
      onClick: handleClick,
      children
    })
  });
}

export { Link, PrefetchLink, commit, createHref, navigate, resolve, toLocation, usePrefetch, useRouter };
//# sourceMappingURL=index.mjs.map
