import { ReactNode } from 'react';
import { History, MemoryHistoryOptions } from 'history';
import type { Options, ResolveView, Route, RouterInstance } from '../types';
import defaultResolve from '../resolve-view';
type Props = {
    children: ReactNode;
    routes: Route[] | Route;
    resolveView?: typeof defaultResolve;
} & Options<ReactNode>;
/**
 * Base Router Component.
 * @group Components
 */
export declare function Router({ router, ...props }: {
    children: ReactNode;
    router: RouterInstance<Route, ReactNode>;
}): import("react/jsx-runtime").JSX.Element;
export declare function createRouter(routes: Route | Route[], history: History, { resolveView, ...options }?: Options<ReactNode> & {
    resolveView?: ResolveView<Route, ReactNode>;
}): RouterInstance<Route, ReactNode>;
/**
 * History mode Router Component.
 * @group Components
 */
export declare function HistoryRouter({ children, routes, ...options }: Props): import("react/jsx-runtime").JSX.Element;
/**
 * Hash mode Router Component.
 * @group Components
 */
export declare function HashRouter({ children, routes, ...options }: Props): import("react/jsx-runtime").JSX.Element;
/**
 * Memory mode Router Component.
 * @group Components
 */
export declare function MemoryRouter({ initialEntries, initialIndex, children, routes, ...options }: Props & MemoryHistoryOptions): import("react/jsx-runtime").JSX.Element;
/**
 * Get Router instance.
 * @group Hooks
 * @returns Router Instance
 */
export declare function useRouter(): RouterInstance<Route, ReactNode>;
export {};
